package tech.sbdevelopment.themeparkaudio;

import co.aikar.commands.PaperCommandManager;
import lombok.Getter;
import org.bukkit.ChatColor;
import tech.sbdevelopment.themeparkaudio.commands.TPAudioCMD;
import tech.sbdevelopment.themeparkaudio.listener.LogoutListener;
import tech.sbdevelopment.themeparkaudio.radio.Playlist;
import tech.sbdevelopment.themeparkaudio.socket.Client;
import org.bukkit.Bukkit;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.java.JavaPlugin;

public final class ThemeParkAudio extends JavaPlugin {
    @Getter
    private static Plugin instance;

    @Getter
    private static PaperCommandManager commandManager;

    @Getter
    private static Client client;

    @Getter
    private static Playlist playlist;

    @Getter
    private static boolean regionSupport = false;

    public void onEnable() {
        Bukkit.getLogger().info("[ThemeParkAudio] Loading...");

        if (Bukkit.getPluginManager().getPlugin("WorldGuard") != null && Bukkit.getPluginManager().getPlugin("Train_Carts") != null) {
            Bukkit.getLogger().info("[ThemeParkAudio] WorldGuard and TrainCarts found! Loading region support...");
            regionSupport = true;
            Bukkit.getPluginManager().registerEvents(new tech.sbdevelopment.themeparkaudio.listener.WGListener(), this);
        } else {
            Bukkit.getLogger().warning("[ThemeParkAudio] WorldGuard not found! Regions won't work...");
        }

        instance = this;

        getConfig().options().copyDefaults(true);
        saveDefaultConfig();

        Bukkit.getLogger().info("[ThemeParkAudio] Connecting with socket...");
        client = new Client(getConfig().getString("socketUrl"));
        client.connect();

        Bukkit.getLogger().info("[ThemeParkAudio] Loading commands and events...");
        commandManager = new PaperCommandManager(this);
        commandManager.enableUnstableAPI("help");

        commandManager.getCommandReplacements().addReplacement("tpcommand", getConfig().getString("command"));
        commandManager.registerCommand(new TPAudioCMD());

        Bukkit.getPluginManager().registerEvents(new LogoutListener(), this);

        if (ThemeParkAudio.getInstance().getConfig().getBoolean("radio")) {
            Bukkit.getLogger().info("[ThemeParkAudio] Loading radio playlist...");
            playlist = new Playlist();
        }

        Bukkit.getLogger().info("[ThemeParkAudio] Plugin is enabled!");
    }

    public void onDisable() {
        client.disconnect();
        instance = null;
        Bukkit.getLogger().info("[ThemeParkAudio] Plugin is disabled!");
    }
}
