package tech.sbdevelopment.themeparkaudio.api.maps;

import java.util.ArrayList;
import java.util.concurrent.ThreadLocalRandom;

/**
 * An {@link ArrayList} with shuffle support.
 *
 * @param <E> The type you want to store
 */
public class SongList<E> extends ArrayList<E> {
    /**
     * Shuffle this List using the Fisher-Yates algorithm
     */
    public void shuffle() {
        ThreadLocalRandom rnd = ThreadLocalRandom.current();
        for (int i = size() - 1; i > 0; i--) {
            int index = rnd.nextInt(i + 1);
            swap(index, i);
        }
    }

    /**
     * Get a random item from this List
     *
     * @return The random element, or null if empty
     */
    public E getRandom() {
        int size = size();
        if (size == 0) return null;
        return get(ThreadLocalRandom.current().nextInt(size));
    }

    /**
     * Swaps two elements in an {@link ArrayList}
     *
     * @param firstIndex The first index
     * @param secondIndex The second index
     */
    private void swap(int firstIndex, int secondIndex) {
        E temp = get(firstIndex);
        set(firstIndex, get(secondIndex));
        set(secondIndex, temp);
    }
}