package tech.sbdevelopment.themeparkaudio.managers;

import lombok.experimental.UtilityClass;
import tech.sbdevelopment.themeparkaudio.ThemeParkAudio;

import java.security.SecureRandom;
import java.util.UUID;
import java.util.WeakHashMap;

@UtilityClass
public class PinManager {
    private static final WeakHashMap<UUID, String> pins = new WeakHashMap<>();

    /**
     * Get the pin of a player
     *
     * @param pUUID The player UUID
     * @return The pin
     */
    public static String getPIN(UUID pUUID) {
        if (pUUID == null) return null;

        if (!pins.containsKey(pUUID)) {
            StringBuilder builder = new StringBuilder();
            SecureRandom random = new SecureRandom();
            for (int i = 0; i < 9; ++i) {
                builder.append(random.nextInt(9));
            }
            String pin = builder.toString();

            pins.put(pUUID, pin);

            ThemeParkAudio.getInstance().getLogger().info("Assigned pin " + pin + " to " + pUUID);

            return pin;
        } else {
            ThemeParkAudio.getInstance().getLogger().info("Pin " + pins.get(pUUID) + " already assigned to " + pUUID);

            return pins.get(pUUID);
        }
    }

    /**
     * Check if a player has a pin
     *
     * @param pUUID The player UUID
     * @return The pin
     */
    public static boolean hasPin(UUID pUUID) {
        return pUUID != null && pins.containsKey(pUUID);
    }

    /**
     * Check if the pin is correct
     *
     * @param pUUID The player UUID
     * @param pin   The pin
     * @return true/false
     */
    public static boolean checkPin(UUID pUUID, String pin) {
        ThemeParkAudio.getInstance().getLogger().info("Checking pin " + pin + " for " + pUUID);

        if (pUUID == null || pin == null || pin.isEmpty()) {
            return false;
        }

        pins.forEach((uuid, p) -> {
            if (pUUID.equals(uuid)) {
                ThemeParkAudio.getInstance().getLogger().info("Pin " + p + " for " + pUUID);
            }
        });

        return pins.containsKey(pUUID) && pin.equals(pins.get(pUUID));
    }

    /**
     * Remove the pin
     *
     * @param pUUID The player UUID
     */
    public static void removePin(UUID pUUID) {
        if (pUUID != null) {
            pins.remove(pUUID);
        }
    }
}
